#!/usr/bin/env bash

CUR_IFS="$IFS"

script_name="$(basename $(realpath $0))"
script_dir="$(dirname $(realpath $0))"

conf_dir="${script_dir}/conf"
snippet_dir="${script_dir}/snippets"

declare -a unsorted_website_arr
declare -a website_arr

log_file="$(mktemp)"

backup_date=$(date +%Y-%m-%d-%H%M)


# =============
# --- Some functions
# =============

clean_up() {


   if [[ -f "$_backup_crontab_file" ]]; then

      echononl "(Re)Install previously saved crontab from '$_backup_crontab_file'.."

      crontab $_backup_crontab_file >> $log_file 2>&1

      if [[ $? -eq 0 ]]; then
         echo_ok
      else
         echo_failed
			error "$(cat $log_file)"
      fi

   fi

   # Perform program exit housekeeping
   rm -f $log_file
   blank_line
   exit $1
}

is_number() {

   return $(test ! -z "${1##*[!0-9]*}" > /dev/null 2>&1);

   # - also possible
   # -
   #[[ ! -z "${1##*[!0-9]*}" ]] && return 0 || return 1
   #return $([[ ! -z "${1##*[!0-9]*}" ]])
}

echononl(){
   echo X\\c > /tmp/shprompt$$
   if [ `wc -c /tmp/shprompt$$ | awk '{print $1}'` -eq 1 ]; then
      echo -e -n "$*\\c" 1>&2
   else
       echo -e -n "$*" 1>&2
   fi
   rm /tmp/shprompt$$
}
echo_done() {
   if $terminal ; then
      echo -e "\033[75G[ \033[32mdone\033[m ]"
   else
      echo " [ done ]"
   fi
}
echo_ok() {
   if $terminal ; then
      echo -e "\033[75G[ \033[32mok\033[m ]"
   else
      echo " [ ok ]"
   fi
}
echo_warning() {
   if $terminal ; then
      echo -e "\033[75G[ \033[33m\033[1mwarn\033[m ]"
   else
      echo " [ warning ]"
   fi
}
echo_failed(){
   if $terminal ; then
      echo -e "\033[75G[ \033[1;31mfailed\033[m ]"
   else
      echo ' [ failed! ]'
   fi
}
echo_skipped() {
   if $terminal ; then
      echo -e "\033[75G[ \033[37mskipped\033[m ]"
   else
      echo " [ skipped ]"
   fi
}
echo_wait(){
   if $terminal ; then
      echo -en "\033[75G[ \033[5m\033[1m..\033[m ]"
   fi
}
fatal (){
   echo ""
   echo ""
   if $terminal ; then
      echo -e "\t[ \033[31m\033[1mFatal\033[m ]: \033[37m\033[1m$*\033[m"
      echo ""
      echo -e "\t\033[31m\033[1m           Script will be interrupted..\033[m\033[m"
   else
      echo "fatal: $*"
      echo "Script will be interrupted.."
   fi
   clean_up 1
}

error(){
   echo ""
   if $terminal ; then
      echo -e "\t[ \033[31m\033[1mFehler\033[m ]: $*"
   else
      echo "Error: $*"
   fi
   echo ""
}

warn (){
   echo ""
   if $terminal ; then
      echo -e "\t[ \033[33m\033[1mWarning\033[m ]: $*"
   else
      echo "Warning: $*"
   fi
   echo ""
}

info (){
   echo ""
   if $terminal ; then
      echo -e "\t[ \033[32m\033[1mInfo\033[m ]: $*"
   else
      echo "Info: $*"
   fi
   echo ""
}

## - Check if a given array (parameter 2) contains a given string (parameter 1)
## -
containsElement () {
   local e
   for e in "${@:2}"; do [[ "$e" == "$1" ]] && return 0; done
   return 1
}

blank_line() {
   if $terminal ; then
      echo ""
   fi
}



# ----------
# - Jobhandling 
# ----------

# - Run 'clean_up' for signals SIGHUP SIGINT SIGTERM
# -
trap clean_up SIGHUP SIGINT SIGTERM

## - 
while IFS='' read -r -d '' _conf_file ; do
   source $_conf_file
   if [[ -n "$WEBSITE" ]] ; then
      unsorted_website_arr+=("${WEBSITE}:$_conf_file")
   fi
   WEBSITE=""
done  < <(find "${conf_dir}" -maxdepth 1 -type f -name "*.conf" -print0)

if [[ ${#unsorted_website_arr} -eq 0 ]]; then
   fatal "No configuration files found in '${script_dir}/conf' or no website configured!"
fi

# - Sort array
# -
IFS=$'\n' website_arr=($(sort <<<"${unsorted_website_arr[*]}"))


# - Reset IFS
# -
IFS=$CUR_IFS



# =============
# --- Some 
# =============

# - Support systemd ?
# -
if [[ "X$(which systemd)" = "X" ]]; then
   SYSTEMD_EXISTS=false
else
   SYSTEMD_EXISTS=true
fi

# - Running in a terminal?
# -
if [[ -t 1 ]] ; then
      terminal=true
else
   terminal=false
fi

#clear

echo ""
echo -e "\033[32m-----\033[m"
echo "Clean up NC Installation"
echo -e "\033[32m-----\033[m"


# Which cloud instance (website) would you like to update
#
source ${snippet_dir}/get-cloud-instance-to-update.sh


# - Reset IFS
# -
IFS=$CUR_IFS


echo ""
echononl "   Include Configuration file.."
if [[ ! -f $conf_file ]]; then
   echo_failed
   fatal "Missing configuration file '$conf_file'"
else
   source $conf_file
   echo_ok
fi
echo ""



# =============
# --- Some checks
# =============

DEFAULT_HTTP_USER="www-data"
DEFAULT_HTTP_USER="www-data"


if [[ -z ${WEBSITE} ]] ; then
   fatal "No website given (parameter 'WEBSITE')"
fi

DEFAULT_WEB_BASE_DIR="/var/www/$WEBSITE"
DEFAULT_PHP_ENGINE="FPM"

[[ -n "$WEB_BASE_DIR" ]] || WEB_BASE_DIR=$DEFAULT_WEB_BASE_DIR

if [[ ! -d ${WEB_BASE_DIR} ]] ; then
   fatal "Web base directory not found (parameter 'WEB_BASE_DIR')"
fi

[[ -n "$PHP_ENGINE" ]] || PHP_ENGINE=$DEFAULT_PHP_ENGINE

if [[ "$DATABASE_TYPE" != "postgres" ]] && [[ "$DATABASE_TYPE" != "mysql" ]]; then
   fatal "Wrong or missing database type (parameter 'DATABASE_TYPE')"
fi

if [[ -z "$DATABASE_NAME" ]]; then
   fatal "Missing database name (parameter 'DATABASE_NAME')"
fi

if [[ "$DATABASE_TYPE" = "mysql" ]] && [[ -z "$MYSQL_CREDENTIALS" ]]; then
   fatal "No Database Credentials for MySQL given (parameter 'MYSQL_CREDENTIALS')"
fi

if [[ "$DATABASE_TYPE" = "postgres" ]]; then
   if [[ -z "$PSQL_USER" ]] || [[ -z "$PSQL_PASS" ]]; then
      fatal "No Database Credentials for PostgreSQL given (parameters: 'PSQL_USER' 'PSQL_PASS'"
   fi
fi


NGINX_IS_ENABLED=false
APACHE2_IS_ENABLED=false

# Get Webservice environment as IS_HTTPD_RUNNING, HTTP_USER, HTTP_GROUP..
#
source ${snippet_dir}/get-webservice-environment.sh


# Check PHP Version
#
source ${snippet_dir}/get-php-major-version.sh


# Get full qualified PHP command
#
source ${snippet_dir}/get-path-of-php-command.sh



# - Get available backup versions 
# -
for _dir in $(ls -d ${WEB_BASE_DIR}/nextcloud-*) ; do
   if [[ "$(basename "$_dir")" =~ ^nextcloud-[0-9]{1,2}\.[0-9]{1,2}\.[0-9]{1,2}\.[0-9]{4} ]]; then
      _version="$(echo "$_dir"| grep -o -E "[0-9]{1,2}\.[0-9]{1,2}\.[0-9]{1,2}")"
      _backup_date="$(echo "$_dir"| grep -o -E "[0-9]{4}-[0-9]{2}-[0-9]{2}-[0-9]{4}")"
      if [[ -d "${WEB_BASE_DIR}/nextcloud-${_version}.${_backup_date}" ]] \
         && [[ -d "${WEB_BASE_DIR}/data-${_version}.${_backup_date}" ]] \
         && [[ -f "${WEB_BASE_DIR}/${DATABASE_NAME}-v${_version}.${_backup_date}.sql" ]]; then

         nextcloud_version_arr+=("$(basename "${_dir}"):${_version}:${_backup_date}")
      fi
   fi
done

if [[ ${#nextcloud_version_arr[@]} -lt 1 ]] ; then
   info "No old version present!"
   clean_up 0
fi


echo ""
echo ""
echo -e "\033[37m\033[1mGoing to delete the folowing old NC installations:..\033[m"
echo ""


for _backup_version in ${nextcloud_version_arr[@]} ; do
   IFS=':' read -a _arr <<< ${_backup_version}
   echo "   [$i] Version ${_arr[1]} from ${_arr[2]}"
   ((i++))
done

echo ""
echo -en "\t\033[33mType upper case 'YES' to continue with deleting ALL old NC installations\033[m: "
read OK
if [[ "$OK" = "YES" ]] ; then
   echo ""
   echo ""
   echo ""
   echo -e "\033[1;32mGoing to delete ALL old NC installations including data directories on \033[1;37m$WEBSITE \033[m"
else
   fatal "Abort by user request - Answer as not 'YES'"
fi


for _backup_version in ${nextcloud_version_arr[@]} ; do
   IFS=':' read -a _arr <<< ${_backup_version}

   echo ""
   echo ""
   echo -e "\033[37m\033[1mGoing to delete the NC Version ${_arr[1]} saved on ${_arr[2]}..\033[m"
   echo ""

   echononl "   Delete SQL file '${DATABASE_NAME}-v${_arr[1]}.${_arr[2]}.sql'"
   if [[ -f "${WEB_BASE_DIR}/${DATABASE_NAME}-v${_arr[1]}.${_arr[2]}.sql" ]]; then

      rm -f "${WEB_BASE_DIR}/${DATABASE_NAME}-v${_arr[1]}.${_arr[2]}.sql" > ${log_file} 2>&1

      if [[ $? -eq 0 ]]; then
         echo_ok
      else
         echo_failed
         error "\n\n$(cat $log_file)"

         echononl "continue anyway [yes/no]: "
         read OK
         OK="$(echo "$OK" | tr '[:upper:]' '[:lower:]')"
         while [[ "$OK" != "yes" ]] && [[  "$OK" != "no" ]] ; do
            echononl "Wrong entry! - repeat [yes/no]: "
            read OK
         done
         [[ $OK = "yes" ]] || fatal "Abbruch durch User"
         blank_line

      fi

   else
      echo_skipped
      warn  "SQL file '${DATABASE_NAME}-v${_arr[1]}.${_arr[2]}.sql' not found!"
   fi

   echononl "   Delete data directory 'data-${_arr[1]}.${_arr[2]}'"
   if [[ -d "${WEB_BASE_DIR}/data-${_arr[1]}.${_arr[2]}" ]]; then

      rm -rf "${WEB_BASE_DIR}/data-${_arr[1]}.${_arr[2]}" > ${log_file} 2>&1
      echo_wait

      if [[ $? -eq 0 ]]; then
         echo_ok
      else
         echo_failed
         error "\n\n$(cat $log_file)"

         echononl "continue anyway [yes/no]: "
         read OK
         OK="$(echo "$OK" | tr '[:upper:]' '[:lower:]')"
         while [[ "$OK" != "yes" ]] && [[  "$OK" != "no" ]] ; do
            echononl "Wrong entry! - repeat [yes/no]: "
            read OK
         done
         [[ $OK = "yes" ]] || fatal "Abbruch durch User"
         blank_line

      fi

   else
      echo_skipped
      warn  "Data directory '${WEB_BASE_DIR}/data-${_arr[1]}.${_arr[2]}' not found!"
   fi

   echononl "   Delete NC install directory 'nextcloud-${_arr[1]}.${_arr[2]}'"
   if [[ -d "${WEB_BASE_DIR}/nextcloud-${_arr[1]}.${_arr[2]}" ]]; then

      rm -rf "${WEB_BASE_DIR}/nextcloud-${_arr[1]}.${_arr[2]}" > ${log_file} 2>&1
      echo_wait

      if [[ $? -eq 0 ]]; then
         echo_ok
      else
         echo_failed
         error "\n\n$(cat $log_file)"

         echononl "continue anyway [yes/no]: "
         read OK
         OK="$(echo "$OK" | tr '[:upper:]' '[:lower:]')"
         while [[ "$OK" != "yes" ]] && [[  "$OK" != "no" ]] ; do
            echononl "Wrong entry! - repeat [yes/no]: "
            read OK
         done
         [[ $OK = "yes" ]] || fatal "Abbruch durch User"
         blank_line

      fi

   else
      echo_skipped
      warn  "NC install directory directory '${WEB_BASE_DIR}/nextcloud-${_arr[1]}.${_arr[2]}' not found!"
   fi


done

clean_up 0
